'' +--------------------------------------------------------------------------+
'' | Cluso's Propeller Operating System - OS boot program:   _OS.BIN          |
'' +--------------------------------------------------------------------------+
'' |  Authors:       (c)2011 by Kwabena W. Agyman (Kye) & J. Moxham (Dr_Acula)|
'' |  Modifications: (c)2012,2013 "Cluso99" (Ray Rodrick)                     |
'' |  License:       MIT License - See end of file for terms of use           |
'' +--------------------------------------------------------------------------+
'' This OS boot program sets up Upper HUB Memory according to the hardware,
''   then calls the the version module (_VER.CMD),
''   which then calls the command module (_CMD.CMD)
''  
'' The command module issues the prompt "SD:>" and waits for a command,
''   then executes the command internally, or externally or by calling
''   an external module (_xxxxxxx.CMD) or a binary file (xxxxxxxx.BIN)
''
'' All external command modules will return to _CMD.CMD after execution.
''
'' Some OS modules are derived from: 
''      KyeDOS v3 (c) 2011 by Kwabena W. Agyman (Kye) & J. Moxham (Dr_Acula)
''      Sphinx    (c) 2009-10 by Michael Park and others
'' Some concepts and code is from Sphinx (Michael Park) & Spinix (Dave Hein).
'' 
'' Special thanks to the following authors for their ideas and MIT code:
''   Michael Park, Dave Hein, Kye, James Moxham, Mike Green, Tomas Rokicki,
''   lonesock, hippy, Jeff Ledger, Roger Williams, Michelli Scales, Andy Schenk.
''   My apologies to any others I may have missed.
'' And of course to Chip Gracey for the fabulous Prop chip and the Spin Interpreter.
'' +--------------------------------------------------------------------------+
'' Modifications by Cluso99...
'' RR20120306   000   Initial module _DOS.CMD
'' RR20120323   004   v0.40 "OS.BIN" 
'' RR20120325   005   v0.50 revised hub layout
'' RR20120619   006   v0.60 use separate _SD_PASM.spin & _SD_FAT.spin
'' RR20130708               fix __module.spin (mountpartition for 2nd file access)
'' RR20130719         v0.70 fix _hwdef.spin for RamBlade SD pins; add info option, improve SD errors
''                          no longer permits running _os.cmd again - delay reqd (was removed)
'' RR20130727               fixed re-executing _os.cmd & _os.bin (calling from RamBlades PropCmd does not work)
'' RR20130728               try outputting SD clocks (PropCMD cannot boot _OS.BIN)
'' RR20130730         v0.72 try resetting card (blockmode etc) using sd_reset.spin (mb_small_spi.spin from kye)
'' RR20130731         v0.73 (use "PropEEP.BIN" to reprogram the eeprom)
'' RR20130802         v0.74 fix _cmd to clear hub on load/run .bin files
'' RR20130802         v0.75 temp fix to use "_RUN.CMD"
'' RR20130903         v0.77 add _getpc & _putpc (incl getcpm -T/B options not wkg)


CON
  _clkmode = _hw#_clkmode_
  _xinfreq = _hw#_xinfreq_

  _nullCharacter                = 0
  _homeCursorCharacter          = 1
  _backspaceCharacter           = 8
  _horizontalTabCharacter       = 9
  _lineFeedCharacter            = 10
  _carriageReturnCharacter      = 13
  _clearScreenCharacter         = 16
  _quoteCharacter               = 34
  _backspaceKeycode             = 200

OBJ
  _hw   : "_hwdef"              'hardware pin definitions etc
  _hub  : "_hubdef3"            'fixed hub definitions ($7FFF... down)

  fat   : "_SD_FAT.spin"        ' "SD-MMC_FATEngine_006.spin"
  taf   : "_SD_FAT.spin"        ' "SD-MMC_FATEngine_006.spin"
  str   : "ASCII0_STREngine.spin"
  std   : "_PCSIO.spin"                 ' pc i/o driver 
  stdin : "_STDIN.spin"                 ' use std input driver
  stdout: "_STDOUT.spin"                ' use std output driver

  sdreset: "sd_reset"           ' Kye's reset block mode etc

dat
csum long   $89                         ' use this to make hub checksum byte @ $0005 = $00 (set to binary byte)

PUB Start | _SDpins, _SIOpins, _SIObaud, cog, z,z0,z1,z2,z3,z4, errorString, n

' waitcnt(clkfreq*5 + cnt)                              ' if we are not cog=0 then delay to permit others to close


{------------------------------------------------------------------------------}
'Determine what hardware we are running on
' preset for defined configuration in _hwdef
  _SIOpins := _hw#_SIOpins
  _SIObaud := _hw#_SIObaud
  _SDpins  := _hw#_SDpins                               
' now check for RamBlade/RamBlade3/TriBlade#2
  z := ina                                              'read the input pins (to check pullups)
  if     (z & _hw#_tb2PU) == _HW#_tb2PU                 'TriBlade#2 ?
    _SDpins  := _hw#_tb2
  elseif (z & _hw#_rb1PU) == _hw#_rb1PU                 'RamBlade ?
    _SIOpins := _hw#_sioRB1                             ' serial pins differ on RB1
    _SDpins := _hw#_rb1
  elseif (z & _hw#_rb3PU) == _hw#_rb3PU                 'RamBlade3 ?
    _SDpins := _hw#_rb3

{------------------------------------------------------------------------------}
'Set hub parameters...
  long[_hub#_HubFree]     := _hub#_HUB_RESERVED         ' set top of hub free
  byte[_hub#_OS_Cogs]     := 0                          ' set resident cogs (not to be stopped)
  byte[_hub#_OS_Clkmode]  := byte[$0004]                ' save the current clockmode
  byte[_hub#_OS_Rows]     := _hw#_SCREENROWS | 0<<7     ' Screen rows & <lf> off (_hub#_OS_LFmask)
  byte[_hub#_OS_Columns]  := _hw#_SCREENCOLS            ' Screen columns
  long[_hub#_OS_Clkfreq]  := long[$0000]                ' save the current clockfreq
' long[_hub#_OS_Seconds]  := 126_230_400 * 3            ' Yr2012 = 4yrs*3
  long[_hub#_OS_DateTime] := 0                          '?????
  long[_hub#_SDpins]      := _SDpins & $1F1F1F1F        
  long[_hub#_SIOpins]     := $0F << 24 | _SIOpins       ' no cog running yet
  long[_hub#_SIObaud]     := _SIObaud                              
  long[_hub#_Hardware]    := 1 << 16 | _SDpins >> 29    ' $0001_000x = cluso's hardware
  str.copy(string("cmd"),_hub#_pBuffer_A)               ' parameters passed between modules

{{===============================================================================
{------------------------------------------------------------------------------}
'Stop all cogs except this one
  cog := cogid                                          ' find this cog
  if cog <> 0
    waitcnt(clkfreq*2 + cnt)                            ' if we are not cog=0 then delay to permit others to close

'Force some clocks to release D0
    outa[_hw#_SDpin_CS]~~                               ' -CS=1
    outa[_hw#_SDpin_CLK]~~                              ' CLK=1
    dira[_hw#_SDpin_CS]~~                               ' -CS=output
    dira[_hw#_SDpin_CLK]~~                              ' CLK=output
    dira[_hw#_SDpin_DI]~                                '  DI=input
    dira[_hw#_SDpin_DO]~                                '  DO=input
    repeat 20                                           ' toggle CLK
      outa[_hw#_SDpin_CLK]~                             '   CLK=0
      waitcnt(clkfreq/1000000 + cnt)                    '   1us
      outa[_hw#_SDpin_CLK]~~                            '   CLK=1
      waitcnt(clkfreq/1000000 + cnt)                    '   1us
    dira[_hw#_SDpin_CLK]~                               ' CLK=input
    dira[_hw#_SDpin_CS]~                                ' -CS=input      
    waitcnt(clkfreq/1000 + cnt)                         '   1ms
      
  repeat n from 0 to 7
    if n <> cog
      cogstop(n)                                        ' stop the cog(s)
===============================================================================}}


{------------------------------------------------------------------------------}
'Start serial...
'                 (inRENDEZVOUS, outRENDEZVOUS, serPins,  baud    )
  cog := std.start(_hub#_StdIn,  _hub#_StdOut,  _SIOpins, _SIObaud) ' start driver, returns cog+1
'             (inRENDEZVOUS )
  stdin.start (_hub#_StdIn  )                           ' start the stub
'             (outRENDEZVOUS)
  stdout.start(_hub#_StdOut )                           ' start the stub

{------------------------------------------------------------------------------}
'Update hub parameters...
  long[_hub#_SIOpins]     := (cog-1) << 24 | _SIOpins
  byte[_hub#_OS_COGS]     := (1 << (cog -1))            ' set resident cog

{------------------------------------------------------------------------------}
  waitcnt(clkfreq/10 + cnt)                             ' small delay
' PrintChar(_clearScreenCharacter)                      ' clear screen
  crlf
  crlf
  PrintStringCR(string("*** Propeller SD card Operating System v0.77 ***"))
  PrintStringCR(string("------------------------------------------------"))
  PrintStringCR(string(" (see 'VER' for acknowledgements)"))


{------------------------------------------------------------------------------}
' Display Locks...
  printString(string(" Locks used: "))
  repeat n from 0 to 7
    if lockset(n)
      printChar(n|$30)
    else
      lockclr(n)
      printChar("-")
  crlf
  

{------------------------------------------------------------------------------}


{------------------------------------------------------------------------------}
'Start SD driver...
  z  :=  long[_hub#_sdPins]
  z0 :=  z        & $1F
  z1 := (z >>  8) & $1F
  z2 := (z >> 16) & $1F
  z3 := (z >> 24) & $1F

{{===============================================================================
{------------------------------------------------------------------------------}
'Try resetting the SD card
  sdreset.start_explicit(z0, z1,  z2, z3)
  printStringCR(string(" Reset SD Card"))
===============================================================================}}

{------------------------------------------------------------------------------}
'Start SD driver...
'                          (DO, CLK, DI, CS, _SD_WP, _SD_CD, _RTC_DAT, _RTC_CLK, -1)
  cog := fat.FATEngineStart(z0, z1,  z2, z3, -1,     -1,     -1,       -1,       -1)
  ifnot (cog)
    reboot

  PrintString(string(" SD Driver Cog = "))
  PrintChar((cog -1) | $30)
  crlf

{------------------------------------------------------------------------------}
'Update hub parameters...
  byte[_hub#_OS_COGS]     |= (1 << (cog -1))            ' set resident cog

{------------------------------------------------------------------------------}
'Display the info (similar to _VER.CMD)
  displayInfo                                           ' display ver params

{------------------------------------------------------------------------------}
'Access the SD card...
  printStringCR(string("i) Booting from SD card"))
  errorString := \fat.mountPartition(0)                 ' mount the sd card
  n := checkError(string("(SD card failure)"), fat.partitionError, errorString) ' if error, display
  if n
    repeat                      '<====================  ' non-recoverable error, loop indefinately 
      printChar("!")
      waitcnt(clkfreq*5 + cnt)
' crlf
  
{------------------------------------------------------------------------------}
'Load/Run "_CMD.CMD"...
' printStringCR(string("i) Loading _CMD.CMD"))
  str.copy(string("cmd"),_hub#_pBuffer_A)               ' parameters passed between modules
  errorString := \fat.bootPartition(string("_CMD.CMD")) ' load/run
  n := checkError(string("_CMD.CMD"), fat.partitionError, errorString) ' if error, display
  
{------------------------------------------------------------------------------}
'Only here on error...
  PrintStringCR(string("ERROR: Rebooting..."))
  waitcnt(clkfreq * 2 + cnt)                            ' delay 
  reboot

{------------------------------------------------------------------------------}

PRI displayInfo | z,z0,z1,z2,z3,z4,i,j,cogs[8]

    PrintString(string(" Hardware: "))
    case long[_hub#_hardware] >> 16  'hardware >> 29
      $0001 : PrintString(string("Cluso's "))
      other : PrintString(string("??? "))
    case long[_hub#_hardware]  'hardware >> 29
      $0001_0001 : PrintString(string("RamBlade"))
      $0001_0002 : PrintString(string("TriBlade#2"))
      $0001_0003 : PrintString(string("RamBlade3"))
      other: PrintString(string("???"))
    crlf

    PrintString(string(" ClockFreq "))
    PrintString(1 + str.integerToDecimal(long[$0],9))
    PrintString(string("Hz, ClockMode $"))
    PrintString(1 + str.integerToHexadecimal(byte[$4],2))
    PrintString(string(", Cog "))
    PrintStringCR(1 + str.integerToDecimal(cogid, 1))

    PrintString(string(" SD card on pins "))
    z  := long[_hub#_sdPins]
    z0 :=  z        & $1F
    z1 := (z >>  8) & $1F
    z2 := (z >> 16) & $1F
    z3 := (z >> 24) & $1F
    PrintString(1 + str.integerToDecimal(z0, 2))
    PrintChar(",")
    PrintString(1 + str.integerToDecimal(z1, 2))
    PrintChar(",")
    PrintString(1 + str.integerToDecimal(z2, 2))
    PrintChar(",")
    PrintString(1 + str.integerToDecimal(z3, 2))
    crlf

    PrintString(string(" Serial  on pins "))
    z  := long[_hub#_sioPins]
    z0 :=  z        & $1F
    z1 := (z >>  8) & $1F
    z2 := (z >> 16) & $1F
    z3 := (z >> 24) & $0F                               ' =$0F if invalid
    z4 :=  long[_hub#_sioBaud]
    PrintString(1 + str.integerToDecimal(z0, 2))
    PrintChar(",")
    PrintString(1 + str.integerToDecimal(z1, 2))
    PrintString(string(", Mode "))
    PrintString(1 + str.integerToDecimal(z2, 2))
    PrintString(string(", Baud "))
    PrintString(1 + str.integerToDecimal(z4, 8))
    PrintString(string(", Cog "))
    if z3 > 7
      PrintChar("?")
    else
      PrintString(1 + str.integerToDecimal(z3, 1))
    crlf

    PrintString(string(" Screen size: "))
    z := byte[_hub#_OS_Columns]
    PrintString(1 + str.integerToDecimal(z, 3))
    PrintString(string(" x "))
    z := byte[_hub#_OS_Rows]
    PrintStringCR(1 + str.integerToDecimal(z, 3))

    PrintString(string(" <LF> is "))
    if byte[_hub#_OS_Rows] & _hub#_LF_MASK                                              
      PrintStringCR(string("ON"))
    else
      PrintStringCR(string("OFF"))

    PrintString(string(" Resident Cogs : "))
    repeat z from 0 to 7
      if (byte[_hub#_OS_Cogs] >> z) & $01 == 1
        PrintString(1 + str.integerToDecimal(z, 1))
        PrintString(string(" "))
    PrintString(string("  This Cog: "))
    PrintStringCR(1 + str.integerToDecimal(cogid, 1))

    PrintString(string(" Cogs available: "))
    j~
    repeat i from 0 to 7
      cogs[i] := cognew(@entry,0) +1                    ' try to start cog(s)
      if cogs[i] 
        j++
    repeat i from 0 to 7
      if cogs[i]
        PrintString(1 + str.integerToDecimal(cogs[i]-1, 1)) ' print avail cog#
        PrintChar(" ")
        cogstop(cogs[i] -1)                             ' stop the cog(s)
    PrintString(string("(="))
    PrintString(1 + str.integerToDecimal(j,1))
    PrintStringCR(string(")"))

PRI checkError(errorMessage, errorNumber, errorString)
  if errorNumber == 0
    return 0
  else
    printString(string("t) Error on file "))
    printString(errorMessage)
    printString(String(" - "))
    printString(str.integerToHexadecimal(errorNumber,2))
    printChar(":")
    printStringCR(errorString)

    return errorNumber

PRI PrintStringCR(pstring)      ' sends string to output devices (vga/serial) with CRLF
    PrintString(pstring)                            
    crlf

PRI PrintString(pstring)        ' sends string to output devices (vga/serial) with no CRLF
    repeat strsize(pstring)
      PrintChar(byte[pstring++])
     
PRI PrintChar(c)                ' sends character to output devices (vga/serial)
    if c <> _lineFeedCharacter
      stdout.out(c)                                     ' send char  to output device
    elseif byte[_hub#_OS_Rows] & _hub#_LF_MASK       
      stdout.out(c)                                     ' send <lf>  to output device

PRI CRLF
    PrintChar(_carriageReturnCharacter)                 ' cr
    PrintChar(_lineFeedCharacter)                       ' lf (optional)

DAT
'' Just a simple pasm program to test if a cog is available
              org       0
entry         jmp       #entry                  ' loops here until cogstop forced!

dat                                                   
{{
+------------------------------------------------------------------------------------------------------------------------------+
|                                                   TERMS OF USE: MIT License                                                  |                                                            
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    | 
|files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}}