'' +--------------------------------------------------------------------------+
'' | Cluso's Propeller Operating System - OS module: _VER  --> _VER.CMD       |
'' +--------------------------------------------------------------------------+
'' |  Authors:       (c)2012,2013 "Cluso99" (Ray Rodrick)                     |
'' |  Modifications:                                                          |
'' |  License:       MIT License - See end of file for terms of use           |
'' +--------------------------------------------------------------------------+
'' Each module is called from, and returns to, the prop binary "_CMD.CMD"
'' This OS module uses an included module "__MODULE.SPIN" to perform most of
''  the housekeeping - see this for history, details and acknowledgements.
''
'' This module......
'' RR20120317     Displays various system info.

'#include "__MODULE.spin"        ' include the common code for OS modules 
#include "__MODULE.spin"

DAT
_ModuleStr    byte      "=== VER ===",0        'module's name (string)
 
PRI executeCommand(stringPointer) | z,z0,z1,z2,z3,z4,i,j,cogs[8]

'' ifnot(str.stringCompareCI(string("ver"), stringPointer))    '<-- module's name
    crlf
    PrintStringCR(string("*** Propeller SD card Operating System v0.7x ***"))
    PrintStringCR(string("------------------------------------------------"))
    PrintStringCR(string(" Derived from KyeDOS..."))
    PrintStringCR(string("   by Kwabena W. Agyeman & J. Moxham & Cluso99 "))
    PrintStringCR(string("    & acknowledgements to M. Park (Sphinx) etc"))
    crlf

    PrintString(string(" Hardware: "))
    case long[_hub#_hardware] >> 16  'hardware >> 29
      $0001 : PrintString(string("Cluso's "))
      other : PrintString(string("??? "))
    case long[_hub#_hardware]  'hardware >> 29
      $0001_0001 : PrintString(string("RamBlade"))
      $0001_0002 : PrintString(string("TriBlade#2"))
      $0001_0003 : PrintString(string("RamBlade3"))
      other: PrintString(string("???"))
    crlf

    PrintString(string(" ClockFreq "))
    PrintString(1 + str.integerToDecimal(long[$0],9))
    PrintString(string("Hz, ClockMode $"))
    PrintString(1 + str.integerToHexadecimal(byte[$4],2))
    PrintString(string(", Cog "))
    PrintStringCR(1 + str.integerToDecimal(cogid, 1))

    PrintString(string(" SD card on pins "))
    z  := long[_hub#_sdPins]
    z0 :=  z        & $1F
    z1 := (z >>  8) & $1F
    z2 := (z >> 16) & $1F
    z3 := (z >> 24) & $1F
    PrintString(1 + str.integerToDecimal(z0, 2))
    PrintChar(",")
    PrintString(1 + str.integerToDecimal(z1, 2))
    PrintChar(",")
    PrintString(1 + str.integerToDecimal(z2, 2))
    PrintChar(",")
    PrintString(1 + str.integerToDecimal(z3, 2))
    crlf

    PrintString(string(" Serial  on pins "))
    z  := long[_hub#_sioPins]
    z0 :=  z        & $1F
    z1 := (z >>  8) & $1F
    z2 := (z >> 16) & $1F
    z3 := (z >> 24) & $0F                               ' =$0F if invalid
    z4 :=  long[_hub#_sioBaud]
    PrintString(1 + str.integerToDecimal(z0, 2))
    PrintChar(",")
    PrintString(1 + str.integerToDecimal(z1, 2))
    PrintString(string(", Mode "))
    PrintString(1 + str.integerToDecimal(z2, 2))
    PrintString(string(", Baud "))
    PrintString(1 + str.integerToDecimal(z4, 8))
    PrintString(string(", Cog "))
    if z3 > 7
      PrintChar("?")
    else
      PrintString(1 + str.integerToDecimal(z3, 1))
    crlf

    PrintString(string(" Screen size: "))
    z := byte[_hub#_OS_Columns]
    PrintString(1 + str.integerToDecimal(z, 3))
    PrintString(string(" x "))
    z := byte[_hub#_OS_Rows]
    PrintStringCR(1 + str.integerToDecimal(z, 3))

    PrintString(string(" <LF> is "))
    if byte[_hub#_OS_Rows] & _hub#_LF_MASK                                              
      PrintStringCR(string("ON"))
    else
      PrintStringCR(string("OFF"))

    PrintString(string(" Resident Cogs : "))
    repeat z from 0 to 7
      if (byte[_hub#_OS_Cogs] >> z) & $01 == 1
        PrintString(1 + str.integerToDecimal(z, 1))
        PrintString(string(" "))
    PrintString(string("  This Cog: "))
    PrintStringCR(1 + str.integerToDecimal(cogid, 1))

    PrintString(string(" Cogs available: "))
    j~
    repeat i from 0 to 7
      cogs[i] := cognew(@entry,0) +1                    ' try to start cog(s)
      if cogs[i] 
        j++
    repeat i from 0 to 7
      if cogs[i]
        PrintString(1 + str.integerToDecimal(cogs[i]-1, 1)) ' print avail cog#
        PrintChar(" ")
        cogstop(cogs[i] -1)                             ' stop the cog(s)
    PrintString(string("(="))
    PrintString(1 + str.integerToDecimal(j,1))
    PrintStringCR(string(")"))

    crlf
    return 0

DAT
'' Just a simple pasm program to test if a cog is available
              org       0
entry         jmp       #entry                  ' loops here until cogstop forced!

dat                                                   
{{
+------------------------------------------------------------------------------------------------------------------------------+
|                                                   TERMS OF USE: MIT License                                                  |                                                            
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    | 
|files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}}