 {==============================================================================}
{
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// General Purpose OS Command Module "_MODULE.spin" 
//
// Derived from SD3.01 FATEngine Demo
// Author:        1/2/2011      Kwabena W. Agyeman     
// Modifications: J. Moxham Dr_Acula to create KyeDOS v3.
// Modifications: Cluso99 (Ray Rodrick) break into modules
// Copyright:     (c) 2011 Kwabena W. Agyeman  (Kye)
//                (c) 2011 J. Moxham           (Dr_Acula)
//                (c) 2012,2013 Ray Rodrick    (Cluso99)
// Licence:       MIT (see end of file for details)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Some OS modules are derived from:
//      KyeDOS v3 (c) 2011 by Kwabena W. Agyman (Kye) & J. Moxham (Dr_Acula)
//      Sphinx    (c) 2009-10 by Michael Park and others
// Some concepts and code is from Sphinx (Michael Park) & Spinix (Dave Hein).
// 
// Special thanks to the following authors for their ideas and MIT code:
//   Michael Park, Dave Hein, Kye, James Moxham, Mike Green, Tomas Rokicki,
//   lonesock, hippy, Jeff Ledger, Roger Williams, Michelli Scales, Andy Schenk.
//   My apologies to any others I may have missed.
// And of course to Chip Gracey for the fabulous Prop chip and the Spin Interpreter.
//
// Special thanks to Bernd Winter for his encouragement, suggestions, testing and donation
//   for helping to get the _xxxCPM commands working (_GETCPM, _PUTCPM, _DIRCPM, etc)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Modifications by Cluso99...
// RR20120318                   Common code included in the OS modules
// RR20120323                   v0.40 change _hubdef
// RR20120325                   v0.50 change _hubdef
// RR20120603                   renamed _MODULE_.SPIN (single underline for homespun)
// RR20120619                   use separate _SD_PASM.spin & _SD_FAT.spin
// RR20130708                   add taf.mountpartition(0) to access a second file (copy & diff were not working)
// RR20130711                   add checkError routine for file errors
// RR20130719                   tidy
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}

CON
  _clkmode = _hw#_clkmode_
  _xinfreq = _hw#_xinfreq_

  _nullCharacter                = 0
  _homeCursorCharacter          = 1
  _backspaceCharacter           = 8
  _horizontalTabCharacter       = 9
  _lineFeedCharacter            = 10
  _carriageReturnCharacter      = 13
  _clearScreenCharacter         = 16
  _quoteCharacter               = 34
  _backspaceKeycode             = 200

OBJ
  _hw   : "_hwdef"                      'hardware pin definitions etc
  _hub  : "_hubdef3"                    'fixed hub definitions ($7FFF... down)

  fat   : "_SD_FAT.spin"        ' "SD-MMC_FATEngine_006.spin"
  taf   : "_SD_FAT.spin"        ' "SD-MMC_FATEngine_006.spin"
  str   : "ASCII0_STREngine.spin"
''std   : "_PCSIO.spin"                 ' pc i/o driver (stays resident, loaded by DOS.BIN)
  stdin : "_STDIN.spin"                 ' use std input driver
  stdout: "_STDOUT.spin"                ' use std output driver

VAR

   byte CommandLine[80] ' complete command line as typed in
   byte LineOfText1[80] ' general purpose string buffer
   byte LineOfText2[80] ' general purpose string buffer
   byte LineOfText3[80] ' general purpose string buffer

   
PUB start | z,z0,z1,z2,z3,z4, errorString, n
{------------------------------------------------------------------------------}
'StdIO is already running. Just start the stubs.
  stdout.start(_hub#_StdOut)                            ' start the stub
  stdin.start(_hub#_StdIn)                              ' start the stub
{------------------------------------------------------------------------------}
'Start the SD driver (later some of this will also be resident)
  z  :=  long[_hub#_SDpins]
  z0 :=  z        & $1F
  z1 := (z >>  8) & $1F
  z2 := (z >> 16) & $1F
  z3 := (z >> 24) & $1F

' printStringCR(string("restarting SD Driver"))          '===========debugging

'                          (DO, CLK, DI, CS, _SD_WP, _SD_CD, _RTC_DAT, _RTC_CLK, -1)
  ifnot( fat.FATEngineStart(z0, z1,  z2, z3, -1,     -1,     -1,       -1,       -1))
'   printStringCR(string("SD driver failed - rebooting"))         '===========debugging
    reboot

  fat.mountPartition(0)                                 ' mount the sd card
  taf.mountPartition(0)                                 ' mount the sd card

  ifnot(fat.partitionMounted)
    PrintString(string("ERROR: Card Not Mounted, stop!"))
    repeat                                              ' LOOP HERE!!!
  ifnot(taf.partitionMounted)
    PrintString(string("ERROR: Card Not Mounted, stop!"))
    repeat                                              ' LOOP HERE!!!
{------------------------------------------------------------------------------}
'Execute the command
  \str.copy(_hub#_pBuffer_A,@commandline)               ' retrieve commandline 
  \str.tokenizeString(@commandline)
  \executeCommand(@commandline)                         ' execute
{------------------------------------------------------------------------------}
'Load/Run "_CMD.CMD"...
  waitcnt(clkfreq / 10 + cnt)                           ' delay
  errorString := \fat.bootPartition(string("_CMD.CMD")) ' load/run
  n := checkError(string("_CMD.CMD"), fat.partitionError, errorString) ' if error, display
{------------------------------------------------------------------------------}
'Only here on error
  PrintStringCR(string("ERROR: Rebooting..."))
  waitcnt(clkfreq * 2 + cnt)                            ' delay
  reboot
{------------------------------------------------------------------------------}

PRI checkError(errorMessage, errorNumber, errorString)
  if errorNumber == 0
    return 0
  else
    printString(string("t) Error on file "))
    printString(errorMessage)
    printString(String(" - "))
    printString(str.integerToHexadecimal(errorNumber,2))
    printChar(":")
    printStringCR(errorString)
    waitcnt(clkfreq*2 + cnt)                            ' wait 2 seconds
    abort errorNumber

PRI GetKey
    result := stdin.in                                  ' wait for an input char
    if result == _backspaceKeycode     ' backspace keycode is 200 but convert to 8
      result := _backspaceCharacter 

PRI PrintHexadecimal(number, length)
    PrintString(str.integerToHexadecimal(number, length))

PRI PrintDecimal(number)               ' print the decimal value, useful for debugging
    PrintString(str.integerToDecimal(number, 10))    

PRI PrintStringCR(pstring)             ' sends string to output devices (vga/serial) with CRLF
    PrintString(pstring)                            
    crlf

PRI PrintString(pstring)               ' sends string to output devices (vga/serial) without CRLF
    repeat strsize(pstring)
      PrintChar(byte[pstring++])
     
PRI PrintChar(c)                       ' sends character to output devices (vga/serial)
    if c <> _lineFeedCharacter
      stdout.out(c)                                     ' send char  to output device
    elseif byte[_hub#_OS_Rows] & _hub#_LF_MASK       
      stdout.out(c)                                     ' send <lf>  to output device
       
PUB CRLF
    PrintChar(_carriageReturnCharacter)                 ' sends <cr> to output device
    PrintChar(_lineFeedCharacter)                       ' sends <lf> to output device

{==============================================================================}
    